@@ -13,8 +13,6 @@ import datastock as ds
 
 # specific
 from . import _generic_check
-from . import _class01_checks as _checks
-from . import _class01_compute as _compute
 
 
 # #############################################################################
@@ -102,17 +100,18 @@ def plot_mesh(
 
     else:
         # possibly time-varying mesh
-        return _plot_mesh_2d_polar(
-            coll=coll,
-            key=key,
-            nmax=nmax,
-            color=color,
-            dax=dax,
-            fs=fs,
-            dmargin=dmargin,
-            dleg=dleg,
-            connect=connect,
-        )
+        raise NotImplementedError()
+        # return _plot_mesh_2d_polar(
+            # coll=coll,
+            # key=key,
+            # nmax=nmax,
+            # color=color,
+            # dax=dax,
+            # fs=fs,
+            # dmargin=dmargin,
+            # dleg=dleg,
+            # connect=connect,
+        # )
 
 
 # #############################################################################
@@ -385,151 +384,151 @@ def _plot_mesh_prepare_2d_tri(
     return grid, grid_bck
 
 
-def _plot_mesh_prepare_polar_cont(
-    coll=None,
-    key=None,
-    k2d=None,
-    RR=None,
-    ZZ=None,
-    ind=None,
-    nn=None,
-):
+# def _plot_mesh_prepare_polar_cont(
+    # coll=None,
+    # key=None,
+    # k2d=None,
+    # RR=None,
+    # ZZ=None,
+    # ind=None,
+    # nn=None,
+# ):
 
-    # ---------------------
-    # sample mesh if needed
+    # # ---------------------
+    # # sample mesh if needed
 
-    # ---------------------
-    # get map of rr / angle
+    # # ---------------------
+    # # get map of rr / angle
 
-    if callable(k2d):
+    # if callable(k2d):
 
-        # check RR
-        if RR is None:
-            msg = (
-                "radius2d / angle2d are callable => provide RR and ZZ!"
-            )
-            raise Exception(msg)
+        # # check RR
+        # if RR is None:
+            # msg = (
+                # "radius2d / angle2d are callable => provide RR and ZZ!"
+            # )
+            # raise Exception(msg)
 
-        # compute map
-        rr = k2d(RR, ZZ)[None, ...]
-        assert rr.ndim == RR.ndim + 1
-        reft = None
-        nt = 1
+        # # compute map
+        # rr = k2d(RR, ZZ)[None, ...]
+        # assert rr.ndim == RR.ndim + 1
+        # reft = None
+        # nt = 1
 
-        if nn is None:
-            nn = 50
+        # if nn is None:
+            # nn = 50
 
-        # create vector
-        rad = np.linspace(np.nanmin(rr), np.nanmax(rr), nn)
+        # # create vector
+        # rad = np.linspace(np.nanmin(rr), np.nanmax(rr), nn)
 
-    else:
-        kn = coll.dobj[coll._which_mesh][key]['knots'][ind]
-        rad = coll.ddata[kn]['data']
-        kb2 = coll.ddata[k2d]['bsplines']
+    # else:
+        # kn = coll.dobj[coll._which_mesh][key]['knots'][ind]
+        # rad = coll.ddata[kn]['data']
+        # kb2 = coll.ddata[k2d]['bsplines']
 
-        if RR is None:
-            km2 = coll.dobj['bsplines'][kb2]['mesh']
-            RR, ZZ = coll.get_sample_mesh(
-                key=km2,
-                res=None,
-                grid=True,
-                mode=None,
-                R=None,
-                Z=None,
-                DR=None,
-                DZ=None,
-                imshow=True,
-            )
+        # if RR is None:
+            # km2 = coll.dobj['bsplines'][kb2]['mesh']
+            # RR, ZZ = coll.get_sample_mesh(
+                # key=km2,
+                # res=None,
+                # grid=True,
+                # mode=None,
+                # R=None,
+                # Z=None,
+                # DR=None,
+                # DZ=None,
+                # imshow=True,
+            # )
 
-        rr = coll.interpolate_profile2d(
-            key=k2d,
-            R=RR,
-            Z=ZZ,
-            grid=False,
-            return_params=False,
-        )[0]
+        # rr = coll.interpolate_profile2d(
+            # key=k2d,
+            # R=RR,
+            # Z=ZZ,
+            # grid=False,
+            # return_params=False,
+        # )[0]
 
-        refr2d = coll.ddata[k2d]['ref']
-        refbs = coll.dobj['bsplines'][kb2]['ref']
-        if refr2d == refbs:
-            reft = None
-            nt = 1
-            rr = rr[None, ...]
-        elif len(refr2d) == len(refbs) + 1 and refr2d[1:] == refbs:
-            reft = refr2d[0]
-            nt = coll.dref[reft]['size']
+        # refr2d = coll.ddata[k2d]['ref']
+        # refbs = coll.dobj['bsplines'][kb2]['ref']
+        # if refr2d == refbs:
+            # reft = None
+            # nt = 1
+            # rr = rr[None, ...]
+        # elif len(refr2d) == len(refbs) + 1 and refr2d[1:] == refbs:
+            # reft = refr2d[0]
+            # nt = coll.dref[reft]['size']
 
-    assert rr.shape[0] == nt
+    # assert rr.shape[0] == nt
 
-    # ----------------
-    # Compute contours
+    # # ----------------
+    # # Compute contours
 
-    contR, contZ = _compute._get_contours(
-        RR=RR,
-        ZZ=ZZ,
-        val=rr,
-        levels=rad,
-    )
+    # contR, contZ = _contours._get_contours(
+        # xx0=RR,
+        # xx1=ZZ,
+        # val=rr,
+        # levels=rad,
+    # )
 
-    # refrad
-    refrad = coll.dobj[coll._which_mesh][key]['knots'][ind]
+    # # refrad
+    # refrad = coll.dobj[coll._which_mesh][key]['knots'][ind]
 
-    return contR, contZ, rad, reft, refrad, RR, ZZ
+    # return contR, contZ, rad, reft, refrad, RR, ZZ
 
 
-def _plot_mesh_prepare_polar(
-    coll=None,
-    key=None,
-    # Necessary for callable radius2d
-    RR=None,
-    ZZ=None,
-):
+# def _plot_mesh_prepare_polar(
+    # coll=None,
+    # key=None,
+    # # Necessary for callable radius2d
+    # RR=None,
+    # ZZ=None,
+# ):
 
-    # --------
-    # prepare
+    # # --------
+    # # prepare
 
-    # create rectangular grid and compute radius at each point
-    k2d = coll.dobj[coll._which_mesh][key]['radius2d']
-    (
-        contRrad, contZrad,
-        rad, reft, refrad,
-        RR, ZZ,
-    ) = _plot_mesh_prepare_polar_cont(
-        coll=coll,
-        key=key,
-        k2d=k2d,
-        RR=RR,
-        ZZ=ZZ,
-        ind=0,
-        nn=None,        # nrad if k2d callable
-    )
+    # # create rectangular grid and compute radius at each point
+    # k2d = coll.dobj[coll._which_mesh][key]['radius2d']
+    # (
+        # contRrad, contZrad,
+        # rad, reft, refrad,
+        # RR, ZZ,
+    # ) = _plot_mesh_prepare_polar_cont(
+        # coll=coll,
+        # key=key,
+        # k2d=k2d,
+        # RR=RR,
+        # ZZ=ZZ,
+        # ind=0,
+        # nn=None,        # nrad if k2d callable
+    # )
 
-    # -----------
-    # contour of angle if angle not None
+    # # -----------
+    # # contour of angle if angle not None
 
-    contRang, contZang, ang, refang = None, None, None, None
-    if len(coll.dobj[coll._which_mesh][key]['shape-c']) == 2:
-        # create rectangular grid and compute radius at each point
-        k2d = coll.dobj[coll._which_mesh][key]['angle2d']
-        (
-            contRang, contZang,
-            ang, _, refang,
-            _, _,
-        ) = _plot_mesh_prepare_polar_cont(
-            coll=coll,
-            key=key,
-            k2d=k2d,
-            RR=RR,
-            ZZ=ZZ,
-            ind=1,
-            nn=None,        # nang if k2d callable
-        )
+    # contRang, contZang, ang, refang = None, None, None, None
+    # if len(coll.dobj[coll._which_mesh][key]['shape-c']) == 2:
+        # # create rectangular grid and compute radius at each point
+        # k2d = coll.dobj[coll._which_mesh][key]['angle2d']
+        # (
+            # contRang, contZang,
+            # ang, _, refang,
+            # _, _,
+        # ) = _plot_mesh_prepare_polar_cont(
+            # coll=coll,
+            # key=key,
+            # k2d=k2d,
+            # RR=RR,
+            # ZZ=ZZ,
+            # ind=1,
+            # nn=None,        # nang if k2d callable
+        # )
 
-    return (
-        contRrad, contZrad, rad, refrad,
-        contRang, contZang, ang, refang,
-        reft,
-    )
+    # return (
+        # contRrad, contZrad, rad, refrad,
+        # contRang, contZang, ang, refang,
+        # reft,
+    # )
 
 
 # #############################################################################
@@ -803,151 +802,151 @@ def _plot_mesh_2d_recttri(
     return dax
 
 
-def _plot_mesh_2d_polar(
-    coll=None,
-    key=None,
-    npts=None,
-    nmax=None,
-    color=None,
-    dax=None,
-    fs=None,
-    dmargin=None,
-    dleg=None,
-    connect=None,
-):
+# def _plot_mesh_2d_polar(
+    # coll=None,
+    # key=None,
+    # npts=None,
+    # nmax=None,
+    # color=None,
+    # dax=None,
+    # fs=None,
+    # dmargin=None,
+    # dleg=None,
+    # connect=None,
+# ):
 
-    # --------------
-    #  Prepare data
+    # # --------------
+    # #  Prepare data
 
-    if nmax is None:
-        nmax = 2
+    # if nmax is None:
+        # nmax = 2
 
-    (
-        contRrad, contZrad, rad, refrad,
-        contRang, contZang, ang, refang,
-        reft,
-    ) = _plot_mesh_prepare_polar(
-        coll=coll,
-        key=key,
-    )
-    refptsr = 'ptsr'
-    nt, nr, nptsr = contRrad.shape
-    if contRang is not None:
-        refptsa = 'ptsa'
-        _, nang, nptsa = contRang.shape
+    # (
+        # contRrad, contZrad, rad, refrad,
+        # contRang, contZang, ang, refang,
+        # reft,
+    # ) = _plot_mesh_prepare_polar(
+        # coll=coll,
+        # key=key,
+    # )
+    # refptsr = 'ptsr'
+    # nt, nr, nptsr = contRrad.shape
+    # if contRang is not None:
+        # refptsa = 'ptsa'
+        # _, nang, nptsa = contRang.shape
 
-    # --------------------
-    # Instanciate Plasma2D
+    # # --------------------
+    # # Instanciate Plasma2D
 
-    coll2 = coll.__class__()
+    # coll2 = coll.__class__()
 
-    # ref
-    coll2.add_ref(
-        key=reft,
-        size=nt,
-    )
-    reft = list(coll2.dref.keys())[0]
-    coll2.add_ref(
-        key=refrad,
-        size=nr,
-    )
-    coll2.add_ref(
-        key=refptsr,
-        size=nptsr,
-    )
+    # # ref
+    # coll2.add_ref(
+        # key=reft,
+        # size=nt,
+    # )
+    # reft = list(coll2.dref.keys())[0]
+    # coll2.add_ref(
+        # key=refrad,
+        # size=nr,
+    # )
+    # coll2.add_ref(
+        # key=refptsr,
+        # size=nptsr,
+    # )
 
-    if contRang is not None:
-        coll2.add_ref(
-            key=refang,
-            size=nang,
-        )
-        coll2.add_ref(
-            key=refptsa,
-            size=nptsa,
-        )
+    # if contRang is not None:
+        # coll2.add_ref(
+            # key=refang,
+            # size=nang,
+        # )
+        # coll2.add_ref(
+            # key=refptsa,
+            # size=nptsa,
+        # )
 
-    # data
-    coll2.add_data(
-        key='radius',
-        data=rad,
-        ref=(refrad,)
-    )
-    coll2.add_data(
-        key='contRrad',
-        data=contRrad,
-        ref=(reft, refrad, refptsr)
-    )
-    coll2.add_data(
-        key='contZrad',
-        data=contZrad,
-        ref=(reft, refrad, refptsr)
-    )
+    # # data
+    # coll2.add_data(
+        # key='radius',
+        # data=rad,
+        # ref=(refrad,)
+    # )
+    # coll2.add_data(
+        # key='contRrad',
+        # data=contRrad,
+        # ref=(reft, refrad, refptsr)
+    # )
+    # coll2.add_data(
+        # key='contZrad',
+        # data=contZrad,
+        # ref=(reft, refrad, refptsr)
+    # )
 
-    if contRang is not None:
-        coll2.add_data(
-            key='angle',
-            data=ang,
-            ref=(refang,)
-        )
-        coll2.add_data(
-            key='contRang',
-            data=contRang,
-            ref=(reft, refang, refptsa)
-        )
-        coll2.add_data(
-            key='contZang',
-            data=contZang,
-            ref=(reft, refang, refptsa)
-        )
+    # if contRang is not None:
+        # coll2.add_data(
+            # key='angle',
+            # data=ang,
+            # ref=(refang,)
+        # )
+        # coll2.add_data(
+            # key='contRang',
+            # data=contRang,
+            # ref=(reft, refang, refptsa)
+        # )
+        # coll2.add_data(
+            # key='contZang',
+            # data=contZang,
+            # ref=(reft, refang, refptsa)
+        # )
 
-    # -----
-    # plot
+    # # -----
+    # # plot
 
-    if contRang is None:
-        return coll2.plot_as_mobile_lines(
-            keyX='contRrad',
-            keyY='contZrad',
-            key_time=reft,
-            key_chan='radius',
-            connect=connect,
-        )
+    # if contRang is None:
+        # return coll2.plot_as_mobile_lines(
+            # keyX='contRrad',
+            # keyY='contZrad',
+            # key_time=reft,
+            # key_chan='radius',
+            # connect=connect,
+        # )
 
-    else:
+    # else:
 
-        daxrad, dgrouprad = coll2.plot_as_mobile_lines(
-            keyX='contRrad',
-            keyY='contZrad',
-            key_time=reft,
-            key_chan='radius',
-            connect=False,
-            inplace=False,
-        )
+        # daxrad, dgrouprad = coll2.plot_as_mobile_lines(
+            # keyX='contRrad',
+            # keyY='contZrad',
+            # key_time=reft,
+            # key_chan='radius',
+            # connect=False,
+            # inplace=False,
+        # )
 
-        daxang, dgroupang = coll2.plot_as_mobile_lines(
-            keyX='contRang',
-            keyY='contZang',
-            key_time=reft,
-            key_chan='angle',
-            connect=False,
-            inplace=False,
-        )
+        # daxang, dgroupang = coll2.plot_as_mobile_lines(
+            # keyX='contRang',
+            # keyY='contZang',
+            # key_time=reft,
+            # key_chan='angle',
+            # connect=False,
+            # inplace=False,
+        # )
 
-        # connect
-        if connect is False:
-            return (daxrad, daxang), (dgrouprad, dgroupang)
+        # # connect
+        # if connect is False:
+            # return (daxrad, daxang), (dgrouprad, dgroupang)
 
-        else:
-            daxrad.setup_interactivity(
-                kinter='inter0', dgroup=dgrouprad, dinc=None,
-            )
-            daxrad.disconnect_old()
-            daxrad.connect()
+        # else:
+            # daxrad.setup_interactivity(
+                # kinter='inter0', dgroup=dgrouprad, dinc=None,
+            # )
+            # daxrad.disconnect_old()
+            # daxrad.connect()
 
-            daxang.setup_interactivity(
-                kinter='inter0', dgroup=dgroupang, dinc=None,
-            )
-            daxang.disconnect_old()
-            daxang.connect()
+            # daxang.setup_interactivity(
+                # kinter='inter0', dgroup=dgroupang, dinc=None,
+            # )
+            # daxang.disconnect_old()
+            # daxang.connect()
 
-            daxrad.show_commands()
-            return daxrad, daxang
+            # daxrad.show_commands()
+            # return daxrad, daxang
